#ifdef USE_ASM

#include <stdlib.h>

#define FALSE 0
#define TRUE 1

longint AvailCPUID (void)
{
    /* Tests for availability of CPUID by flipping ID bit eflags */
    longint f=FALSE;
    __asm
        {
        pushfd
        pop eax
        mov ebx,eax
        xor eax,0x00200000
        push eax
        popfd
        pushfd
        pop eax
        push ebx
        popfd
        xor eax,ebx
        mov f,eax
        }

    return f;
}

longint FirstCPUID (char *s)
{
    longint Level;
    // Find processor manufacturer and number of supported CPUID modes
    __asm
        {
        xor eax,eax
        cpuid
        mov Level,eax
        mov esi,s
        mov dword ptr [esi],ebx
        mov dword ptr 4[esi],edx
        mov dword ptr 8[esi],ecx
        }
    s [12] = 0;
    return Level;
}

longint FirstAMDCPUID (void)
{
    longint Level;
    // Finds number of supported AMD extended CPUID modes
    __asm
        {
        mov eax,80000000H
        cpuid
        mov Level,eax
        }
    return Level;
}

unsigned longint VersionCPUID (void)
{
    unsigned longint Features;
    // Finds features of CPU
    __asm
        {
        mov eax,1
        cpuid
        mov Features,edx
        }
    return Features;
}

longint FamilyCPUID (void)
{
    unsigned longint Family;
    // Finds family of CPU
    __asm
        {
        mov eax,1
        cpuid
        mov Family,eax
        }
    return Family;
}

unsigned longint AMDVersionCPUID (void)
{
    unsigned longint Features;
    // Finds AMD extended features of CPU
    __asm
        {
        mov eax,80000001H
        cpuid
        mov Features,edx
        }
    return Features;
}

bool CheckCPUVender(char* Vender)
{
	long int tmpVender[3];
	__asm
	{
		xor eax, eax
		cpuid
		mov [tmpVender], ebx
		mov [tmpVender + 4], edx
		mov [tmpVender + 8], ecx
	}
	return(strncmp((char*) tmpVender, Vender, strlen(Vender)) == 0);
}

unsigned longint cpuFeatures (unsigned longint eaxval = 1)
{
	unsigned longint features;
	__asm
	{
		mov eax, eaxval			;set request type to a feature list from processor
		cpuid				;send request
		lea ebx, features	
		mov [ebx], edx		;save value
	}

	//cpufeatures = features; 
	return features;
}

#else

longint AvailCPUID()
{
	return(true);
}

longint FamilyCPUID()
{
	return(6);
}

longint VersionCPUID()
{
	return(-1);
}

longint AMDVersionCPUID()
{
	return(-1);
}

bool CheckCPUVender(char* tmp)
{
	return(true);
}

longint FirstCPUID(char* tmp)
{
	return(1);
}

longint FirstAMDCPUID()
{
	return(0x80000001);
}

unsigned longint cpuFeatures(unsigned longint tmp = -1)
{
	return((unsigned) -1);
}

#endif

longint isPentium (void)
{
   unsigned longint Features;
    // Returns TRUE if processor is a Pentium or compatible.
    if (!AvailCPUID ()) return FALSE;       // no CPUID
    if (FamilyCPUID () < 5) return FALSE;   // must be 486
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    return TRUE;
}

longint isPentiumMMX (void)
{
   unsigned longint Features;
    // Returns TRUE if processor is a Pentium with MMX or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

longint isPentiumPro (void)
{
    unsigned longint Features;
    // Returns TRUE if processor is a Pentium Pro or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    return TRUE;
}

longint isPentiumII (void)
{
    unsigned longint Features;
    // Returns TRUE if processor is a Pentium II or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

longint isPentiumIII (void)
{
   unsigned longint Features;
    // Returns TRUE if processor is a Pentium III or compatible.
    // Returns FALSE if Pentium III with no operating system support
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x2000000)) return FALSE; // no Streaming SIMD
    //if (!(isSSEOS ())) return FALSE; // no Streaming SIMD operating system support
    return TRUE;
}

longint isAMDK6 (void)
{
    char Brand [64];
    unsigned longint Features;
    // Returns TRUE if processor is an AMD K6 with MMX or compatible.
    // Returns FALSE if processor is a Pentium MMX, even though this
    // is a compatible processor. Use isPentiumMMX for that.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;    // not Pentium class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

longint isAMDK62 (void)
{
    char Brand [64];
    unsigned longint Features;
    // Returns TRUE if processor is an AMD K6 with MMX and 3DNow! or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;    // not Pentium class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x80000000)) return FALSE; // no 3DNow!
    return TRUE;
}

longint isAthlon (void)
{
    char Brand [64];
    unsigned longint Features;
    // Returns TRUE if processor is an AMD Athlon with MMX and Advanced 3DNow! or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;    // not Pentium Pro class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x400000)) return FALSE; // no K7 MMX
    if (!(Features & 0x80000000)) return FALSE; // no 3DNow!
    if (!(Features & 0x40000000)) return FALSE; // no Advanced 3DNow!
    return TRUE;
}


#define CHECKFEATURE_ERROR  0x10000001	/* cpu feature check had a problem, call cpuFeatures */
#define CHECKFEATURE_X87FPU 0x00000001  /* cpu has x87 fpu */
#define CHECKFEATURE_VMODEE 0x00000002  /* cpu has Virtual 8086 mode enhancement */
#define CHECKFEATURE_DEBEXT 0x00000004  /* cpu has debugging extensions */
#define CHECKFEATURE_PGSEXT	0x00000008  /* cpu has page extensions */
#define CHECKFEATURE_TSCOUN	0x00000010  /* cpu has time stamp counter */
#define CHECKFEATURE_XXMSRS	0x00000020  /* cpu has RDMSR/WRMSR inst support */
#define CHECKFEATURE_PHAEXT	0x00000040  /* cpu has physical address extension */
#define CHECKFEATURE_MCHEXT	0x00000080  /* cpu has machine check exception */
#define CHECKFEATURE_CMPXCH	0x00000100  /* cpu has 8 byte cmp and xchg inst support */
#define CHECKFEATURE_APICON	0x00000200  /* cpu has APIC on chip */
#define CHECKFEATURE_SYSXXX	0x00000800  /* cpu has SYSENTER/SYSLEAVE */
#define CHECKFEATURE_MTRREG	0x00001000  /* cpu has memory type range registers */
#define CHECKFEATURE_PTEBIT	0x00002000  /* cpu has global PTE bit */
#define CHECKFEATURE_MCKARC	0x00004000  /* cpu has machine check architexture */
#define CHECKFEATURE_CMOVIN	0x00008000  /* cpu has CMOV inst support */
#define CHECKFEATURE_PGATTT	0x00010000  /* cpu has page attribute table */
#define CHECKFEATURE_PGSEX2	0x00020000  /* cpu has page size extension */
#define CHECKFEATURE_SERNUM	0x00040000  /* cpu has processor serial number */
#define CHECKFEATURE_CFLUSH	0x00080000  /* processor supports CFLUSH inst */
#define CHECKFEATURE_DBSTOR	0x00200000  /* cpu has debug store */
#define CHECKFEATURE_ACPICT	0x00400000  /* ACPI on processor */
#define CHECKFEATURE_MMXCMP	0x00800000  /* cpu has MMX technology */
#define CHECKFEATURE_FXXSTO	0x01000000	/* cpu supports FXSAVE/FXRSTORE inst */
#define CHECKFEATURE_SSEEXT	0x02000000  /* cpu has SSE extensions */
#define CHECKFEATURE_SSE2EX 0x04000000  /* cpu has sse2 extensions */
#define CHECKFEATURE_SSNOOP	0x08000000  /* cpu supports system snoop */
#define CHECKFEATURE_PTHERM	0x10000000  /* cpu has process thermomitor */

#define CHECKFEATURE_SSE3EX 0x00000001  /* SSE 3 Extension EXTENDED FLAG !!! */
#define CHECKFEATURE_SSSE3EX 0x00000200  /* Intel Core 2 SSSE 3 */

unsigned longint cpuCheckFeature (unsigned longint feature, bool extended = false)
{
	if(cpuFeatures() & feature) /* feature exists, good- tell user */
		return feature;

	return 0;
}

bool HasSSE()
{
	if (isPentiumIII == 0)
	{
		return(false);
	}
	if (cpuCheckFeature(CHECKFEATURE_SSEEXT) == CHECKFEATURE_SSEEXT)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}

bool HasSSE2()
{
	if (!HasSSE())
	{
		return(false);
	}
	if (cpuCheckFeature(CHECKFEATURE_SSE2EX) == CHECKFEATURE_SSE2EX)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}

bool HasSSE3()
{
	if (!HasSSE2())
	{
		return(false);
	}
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
	if (cpuCheckFeature(CHECKFEATURE_SSE3EX, true) == CHECKFEATURE_SSE3EX)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}

bool HasSSSE3()
{
	if (!HasSSE3()) return(false);
	if (FirstAMDCPUID() < 0x80000001) return(false);
	if (cpuCheckFeature(CHECKFEATURE_SSSE3EX, true) != CHECKFEATURE_SSSE3EX) return(false);
	return(true);
}

bool isIntel()
{
	return(CheckCPUVender("GenuineIntel"));
}

bool isAMD()
{
	return(CheckCPUVender("AuthenticAMD"));
}

void printCpuType()
{
	if (isIntel())
	{
		if (HasSSSE3())
		{
			qprintf("Intel Core 2");
		}
		else if (HasSSE3())
		{
			qprintf("Intel Pentium 4 Prescott");
		}
		else if (HasSSE2())
		{
			qprintf("Intel Pentium 4");
		}
		else if (isPentiumIII())
		{
			qprintf("Intel Pentium III");
		}
		else if (isPentiumII())
		{
			qprintf("Intel Pentium II");
		}
		else if (isPentiumPro())
		{
			qprintf("Intel Pentium Pro");
		}
		else if (isPentiumMMX())
		{
			qprintf("Intel Pentium MMX");
		}
		else if (isPentium())
		{
			qprintf("Intel Pentium");
		}
		else
		{
			qprintf("Intel 80486");
		}
			
	}
	else if (isAMD())
	{
		if (HasSSE2())
		{
			qprintf("AMD Athlon 64");
		}
		else if (isAthlon())
		{
			qprintf("AMD Athlon");
		}
		else if (isAMDK62())
		{
			qprintf("AMD K6 2");
		}
		else if (isAMDK6())
		{
			qprintf("AMD K6");
		}
		else
		{
			qprintf("AMD K5");
		}
	}	
	else
	{
		qprintf("Unknown CPU Type");
	}
}

void printCpuFeatures()
{
	unsigned longint Features, AMDFeatures;
	Features = VersionCPUID ();
	AMDFeatures = AMDVersionCPUID ();
	if (Features & 0x800000) qprintf("MMX ");
    if (AMDFeatures & 0x80000000) qprintf("3dNow ");
    if (AMDFeatures & 0x40000000) qprintf("Adv3dNow ");
	if (HasSSE()) qprintf("SSE ");
	if (HasSSE2()) qprintf("SSE2 ");
	if (HasSSE3()) qprintf("SSE3 ");
	if (HasSSSE3()) qprintf("SSSE3 ");
}

