#ifdef STRING_KEEP_ALLOC
#define STRING_BYTEVAL Bytes * STRING_KEEP_ALLOC
#else
#define STRING_BYTEVAL Bytes
#endif

class basestring
{
protected:
	size_t len;
	bool alloc;

public:
	char* buffer;

protected:
	inline void savefree()
	{
		if (alloc)
		{
			freel(buffer);
			alloc = false;
		}
	}

	inline char* savealloc(const size_t Bytes)
	{
		char* tmpBuffer;

		if (Bytes == 1)
		{
			alloc = false;
			return(EmptyString);
		}

		alloc = true;
		tmpBuffer = (char*) mallocl(STRING_BYTEVAL, "String");

		return(tmpBuffer);
	}

	inline char* saverealloc(void* Pointer, const size_t Bytes)
	{
		char* tmpBuffer;

		if (Bytes == 1)
		{
			savefree();
			alloc = false;
			return(EmptyString);
		}

		alloc = true;
		if (Pointer != EmptyString)
		{
			tmpBuffer = (char*) reallocl(Pointer, STRING_BYTEVAL);
		}
		else
		{
			tmpBuffer = (char*) mallocl(STRING_BYTEVAL, "String-re");
		}

		return(tmpBuffer);
	}

public:

	void reset()
	{
		this->~basestring();
		ZeroMemory(this, sizeof(basestring));
		buffer = EmptyString;
	};

	basestring(const char* pInit = NULL)
	{
		//printf("Konstruktor\n");
		if (pInit)
		{
			len = strlen(pInit);
			buffer = savealloc(len + 1);
			CopyMemory(buffer, pInit, len + 1);
		}
		else
		{
			buffer = EmptyString;
			len = 0;
			alloc = false;
		}
	}

	~basestring()
	{
		//printf("Destruktor\n");
		savefree();
		alloc = false;
	}

	basestring(const basestring& strInit)
	{
		//printf("Kopier-Konstruktor\n");
		if (strInit.alloc)
		{
			len = strInit.len;
			buffer = savealloc(len + 1);
			CopyMemory(buffer, strInit.buffer, len + 1);
		}
		else
		{
			buffer = EmptyString;
			len = 0;
			alloc = false;
		}
	}

#define STRING_INCLUDE_IN_CLASS
#define STRING_INCLUDE_TYPE basestring
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE

};

class logstring : public basestring
{
public:
#ifdef QON_ENABLE_LOG_STRING
#include "../malloclog/newlog.h"
#endif

#define STRING_INCLUDE_TYPE logstring
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE
};

class string : public basestring
{
protected:
	size_t allocated;
	char* realbuffer;

	inline void savefree()
	{
		if (alloc)
		{
#ifndef STRING_KEEP_ALLOC
			freel(buffer);
#endif
			alloc = false;
		}
	}

	inline char* savealloc(const size_t Bytes)
	{
		char* tmpBuffer;

		if (Bytes == 1)
		{
			alloc = false;
			return(EmptyString);
		}

		alloc = true;
#ifdef STRING_KEEP_ALLOC
#ifdef STRING_KEEP_ALLOC_LOG
		stringKeepAllocTry++;
#endif
		if (allocated >= Bytes)
		{
#ifdef STRING_KEEP_ALLOC_LOG
			stringKeepAllocWorked++;
#endif
			return(realbuffer);
		}
		if (allocated)
		{
			freel(realbuffer);
		}
#endif
		tmpBuffer = (char*) mallocl(STRING_BYTEVAL, "String");

#ifdef STRING_KEEP_ALLOC
		realbuffer = tmpBuffer;
		allocated = Bytes * STRING_KEEP_ALLOC;
#endif
		
		return(tmpBuffer);
	}

	inline char* saverealloc(void* Pointer, const size_t Bytes)
	{
		char* tmpBuffer;

		if (Bytes == 1)
		{
			savefree();
			alloc = false;
			return(EmptyString);
		}

		alloc = true;
#ifdef STRING_KEEP_ALLOC
#ifdef STRING_KEEP_ALLOC_LOG
		stringKeepAllocTry++;
#endif
		if (allocated >= Bytes)
		{
#ifdef STRING_KEEP_ALLOC_LOG
			stringKeepAllocWorked++;
#endif
			return(realbuffer);
		}
#endif

		if (Pointer != EmptyString)
		{
			tmpBuffer = (char*) reallocl(Pointer, STRING_BYTEVAL);
		}
		else
		{
#ifdef STRING_KEEP_ALLOC
			if (allocated)
			{
				freel(realbuffer);
			}
#endif
			tmpBuffer = (char*) mallocl(STRING_BYTEVAL, "String-re");
		}

#ifdef STRING_KEEP_ALLOC
		realbuffer = tmpBuffer;
		allocated = Bytes * STRING_KEEP_ALLOC;
#endif
		
		return(tmpBuffer);
	}

public:

	void reset()
	{
#ifdef STRING_KEEP_ALLOC
		buffer = EmptyString;
		len = 0;
		alloc = false;
#else
		this->~string();
		ZeroMemory(this, sizeof(string));
		buffer = EmptyString;
#endif
	}


	string(const char* pInit = NULL)
	{
		//printf("Konstruktor\n");
#ifdef STRING_KEEP_ALLOC
			allocated = 0;
#endif
		if (pInit)
		{
			len = strlen(pInit);
			buffer = savealloc(len + 1);
			CopyMemory(buffer, pInit, len + 1);
		}
		else
		{
			buffer = EmptyString;
			len = 0;
			alloc = false;
		}
	}

	~string()
	{
		//printf("Destruktor\n");
		savefree();
		alloc = false;
#ifdef STRING_KEEP_ALLOC
		if (allocated)
		{
			freel(realbuffer);
		}
		allocated = 0;
#endif
	}

	string(const string& strInit)
	{
		//printf("Kopier-Konstruktor\n");
#ifdef STRING_KEEP_ALLOC
			allocated = 0;
#endif
		if (strInit.alloc)
		{
			len = strInit.len;
			buffer = savealloc(len + 1);
			CopyMemory(buffer, strInit.buffer, len + 1);
		}
		else
		{
			buffer = EmptyString;
			len = 0;
			alloc = false;
		}
	}

#define STRING_INCLUDE_TYPE string
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE
#undef STRING_INCLUDE_IN_CLASS
};

#define STRING_INCLUDE_AFTER_CLASS

#define STRING_INCLUDE_TYPE string
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE

#define STRING_INCLUDE_TYPE basestring
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE

#define STRING_INCLUDE_TYPE logstring
#include "stringinc.h"
#undef STRING_INCLUDE_TYPE

#undef STRING_INCLUDE_AFTER_CLASS

extern inline void ucase(char* strStr);

#undef STRING_BYTEVAL
